#' Combines source distributions into the resultant integrated distribution
#' using the average of distribution probabilities. The latter one
#' is called an \emph{average} probability distribution.
#' 
#' The probability of the true value to be equal \emph{z} is computed as 
#' \emph{(p1[z]+p2[z]+...+pn[z])/n}, where \emph{p1...pn} are probabilities 
#' from \emph{x} and \emph{n} is the number of source distributions.
#' 
#' @title Average distribution
#' @param x an R object with source distributions
#' @param ... other arguments passed to methods
#' @return A data frame with the 'table-based' distribution, where every row
#' contains an outcome in the random variable range and its probability in the
#' integrated distribution.
#' @examples
#' # Average distribution of two 'table-based' distributions
#' # S3 method for class "distribution_table"
#' pdf1 <- c(0.25, 0.75)
#' pdf2 <- c(0.5, 0.5)
#' bins <- c(1, 2)
#' average(integrate(bins, list(pdf1, pdf2)))
#' @export
average <- function(x, ...) {
  UseMethod("average", x)
}

#' @export
average.distribution_table <- function(x, ...) {
  return (data.frame(
    x = x$pdfs[, 1],
    prob = as.vector(
      apply(x$pdfs[, -1], 1, function(r)(sum(r) / length(r))))))
}
