# Defines distributions in the table format.

# Serves as a constructor for the distribution_table class.
# 
# vals sets right borders of bins. A left border is equal to the previous
# right border value. For the first value, the left border equals -Inf.
# The probability from the last bin to +Inf always equals 0. 
# 
# @title Distribution tables
# @param vals bins
# @param probabilities a list of probabilities, assigned for bins
# @return An instance of the distribution_table class.
# Throws exception when  vectors are not of the same length,
# when 'vals' argument is not a vector with at least 2 elements,
# when 'probabilities' is not a list,
# when items in 'probabilities' are not vectors with at least 2 elements.
distribution_table <- function(vals, probabilities){
  check_argument(vals, "'vals'")

  if (!is.vector(probabilities))
    throw_arg_exception("'probabilities'")
  sapply(probabilities,  function(p)(check_argument(p, "'probabilities'")))

  if (is.null(names(probabilities)))
    names(probabilities) <- sapply(
      seq(1 : length(probabilities)),
      function(index)(paste("P", index, sep = "")))

  all_args <- append(list("x" = vals), probabilities)

  if (!all(tail(duplicated(sapply(all_args, length)), -1)))
    stop("Length of vector arguments should match")

  object <- list(
    pdfs = data.frame(do.call(cbind, all_args)))

  class(object) <- "distribution_table"

  return (invisible(object))
}

#' Test of an object being interpretable as a distribution table.
#' 
#' @title Distribution tables
#' @param obj an object to be tested
#' @return The method returns TRUE if its argument inherits from the class 
#' "distribution_table", and FALSE otherwise.
#' @export
is.distribution_table <- function(obj){
  return (inherits(obj, "distribution_table"))
}

# Checks whether a value is numeric and a vector with at least two items.
# 
# Throws exception, if a supplied value is not numeric or is not a vector 
# with at least two items.
# 
# @title Argument check
# @param arg a supplied value
# @param label a label of the argument
check_argument <- function(arg, label){
  if (!is.numeric(arg) || !(length(arg) > 1))
    throw_arg_exception(label)
}

# Throws argument type exception.
# 
# @title Argument type exception
# @param label a label of the argument
throw_arg_exception <- function(label){
  stop(paste(label, ": bad argument type", sep = ""))
}
