#' Combines distributions into an aggregated discrete distribution using several 
#' methods.
#' 
#' Integration is done for the discrete prior stochastic estimates. Every
#' prior has the same range specified in \emph{vals}. Prior distribution
#' can be specified in the 'table-based' format or in the 'cdf-based' format. A 
#' 'table-based' distribution gives a probability to every outcome in the range. A
#' 'cdf-based' distribution assigns a probability for every bin, whose middle
#' point is an outcome in the range and bin's borders are defined between the 
#' range points.
#'
#' @title Integration of distributions
#' @param vals a discrete ordered range of the random variables
#' @param pdfs probability distributions in the 'table-based' format
#' @param cdfs probability distributions in the 'cdf-based' format
#' @return An object of the distribution_table class.
#' @note Source distributions are integrated using the posterior integration 
#' method [Kryazhimskiy, 2013] and using simple averaging of all input 
#' distributions.
#' @seealso \code{\link{product}}, \code{\link{average}}, 
#' \code{\link{summary.distribution_table}}
#' @examples
#' # Integration of the two 'table-based' distributions
#' range <- c(1, 2, 3)
#' result <- integrate(range, list("m1" = c(0.3, 0.6, 0.1), "m2" = c(0.2, 0.4, 0.4)))
#' summary(result)
#' print(result)
#' 
#' # Integration of the 'table-based' and 'cdf-based' distributions
#' integrate(
#'    forest_npp90[, 1], 
#'    as.list(forest_npp90["LEA_Tundra"]),
#'    list("DGVM_Tundra" = function(x)(pnorm(x, mean = 202, sd = 52))))
#' 
#' # Integration of the 'cdf-based' distributions
#' integrate(
#'    c(1, 2),
#'    cdfs = list(function(x)(punif(x, min = 0.5, max = 2.5)),
#'                function(x)(punif(x, min = 0.5, max = 2.5))))
#' @references Kryazhimskiy, A.V. (2013). Posterior integration of independent 
#' stochastic estimates. IIASA Interim Report. IR-13-006.
#' @export
integrate <- function(vals, pdfs, cdfs){
  if (missing(cdfs)) {
    return (distribution_table(vals, pdfs))
  } else {
    h <- (vals[2] - vals[1]) / 2

    pdfs2 <- lapply(
      cdfs,
      function(cdf)(
        sapply(vals, function(x)(cdf(x + h) - cdf(x - h)))))

    if (missing(pdfs)) {
      return (distribution_table(vals, pdfs2))
    } else {
      return (distribution_table(vals, append(pdfs, pdfs2)))
    }
  }
}
