#' Provides descriptive statistics of the source and integrated distributions.
#' 
#' Descriptive statistics include distribution mean and standard deviation. 
#' Source distributions are combined into the product and average integrated
#' distributions.
#' 
#' @title Descriptive statistics
#' @param x an R object with source distributions
#' @param ... other arguments passed to methods
#' @return A data frame with means and standard deviations. The the last two 
#' columns "Product" and "Average" contain estimates for the corresponding 
#' integrated distributions.
#' @seealso \code{\link{product}}, \code{\link{average}}
#' @examples
#' # S3 method for class "distribution_table"
#' pdf1 <- c(0.75, 0.25)
#' pdf2 <- c(0.75, 0.25)
#' bins <- c(1, 2)
#' statistics(integrate(bins, list(pdf1, pdf2)))
#' @export
statistics <- function(x, ...) {
  UseMethod("statistics", x)
}

#' @export
statistics.distribution_table <- function(x, ...) {
  probs <- cbind(
    x$pdfs[, -1],
    product(x)[, -1],
    average(x)[, -1])

  take_mean <- function(vals)(
    colSums(apply(probs, 2, function(col)(col * vals))))

  mean <- take_mean(x$pdfs[, 1])
  std <- sqrt(take_mean(x$pdfs[, 1] ^ 2) - mean ^ 2)

  stats <- as.data.frame(rbind(mean, std))

  colnames(stats) <- append(names(x$pdfs[, -1]), c("Product", "Average"))

  return (stats)
}
